def pairwise(iterable):
    "s -> (s0, s1), (s2, s3), (s4, s5), ..."
    a = iter(iterable)
    return zip(a, a)


def quote(s):
    return "\"" + s + "\""


def unquote(s):
    if s.startswith('"'):
        s = s[1:]

    if s.endswith('"'):
        s = s[:-1]

    return s



class GPS(object):
    def __init__(self, gps_raw):
        gps_parsed = gps_raw.split(",")
        # from pprint import pprint
        # pprint(gps_parsed)
        self.latitude = gps_parsed[1].strip()
        self.longitude = gps_parsed[2].strip()



from em73xx import Modem
from .mock_serial import MockSerial
import unittest2

dev = "/dev/ttyACM0"
bps = 460800
pin = None
debug = False

message_senders = [
    "+420775123456",
    "+420775123457",
    "+420775123458"
]

test_responses = []

text_mode_AT = "AT+CMGF=1"
all_sms_AT = "AT+CMGL=\"ALL\""

for i, sender in enumerate(message_senders):
    # flake8: noqa
    test_responses.append('+CMGL: 1,"REC READ","%s",,"21/10/16,12:00:00+08"\r\n' % (sender))
    test_responses.append('%s\r\n' % str(i))


class GetSmsTest(unittest2.TestCase):
    def __init__(self, *args, **kwargs):
        serial = MockSerial(test_responses)
        self.modem = Modem(dev, bps, pin, serial, debug)
        self.received_sms_messages = self.modem.getSMS()
        super(GetSmsTest, self).__init__(*args, **kwargs)

    def test_correct_number_sms(self):
        self.assertEqual(3, len(self.received_sms_messages))

    def test_correct_message(self):
        for i, sms in enumerate(self.received_sms_messages):
            self.assertEqual(sms.message, str(i))

    def test_correct_sender(self):
        for i, sms in enumerate(self.received_sms_messages):
            self.assertEqual(sms.sender, message_senders[i])

    def test_AT_commands(self):
        self.assertEqual(text_mode_AT, self.modem.device.inputs[0].strip())
        self.assertEqual(all_sms_AT, self.modem.device.inputs[1].strip())



import pkgutil
import unittest2


def all_names():
    for _, modname, _ in pkgutil.iter_modules(__path__):
        if modname.startswith('test_'):
            yield 'em73xx.test.' + modname


def all():
    return unittest2.defaultTestLoader.loadTestsFromNames(all_names())



class MockSerial(object):
    def __init__(self, response):
        self.response = response
        self.inputs = []

    def write(self, message):
        self.inputs.append(message)

    def readlines(self):
        return self.response



import unittest2

# need to ensure that the package exports the following symbols:
#
# - Modem


class TestImports(unittest2.TestCase):

    def test_import(self):
        # flake8: noqa
        import em73xx
        dir_em73xx = dir(em73xx)
        self.assertIn("Modem", dir_em73xx)



from em73xx import Modem
from .mock_serial import MockSerial
import unittest2

dev = "/dev/ttyACM0"
bps = 460800
pin = None
debug = False

expected_latitude = "49.195407 N"
expected_longitude = "16.606729 E"

# flake8: noqa
stub_gps_data = [
    '\r\n',
    '+XLCSLSR: 2, %s, %s, 231.177947, 60.431427, 25.304975,149, 81.534980,67,2016/10/21,21:33:31,0,1,320.07,0.72,-0.66,2.73,0.90,509628,509628.47,,,3.50,2.25,2.50,104.59,65.52,81.53,,,1,1919,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,6,81.04,285.14,32.00,1,,,,13 ,74.04,307.16,0.00,1,,,,10 ,65.03,85.04,32.00,1,,,,28 ,65.03,231.12,26.00,1,,,,8 ,39.02,318.16,24.00,1,,,,1 ,39.02,48.02,26.00,1,,,,9 \r\n' % (expected_latitude, expected_longitude),
    '\r\n',
    'OK\r\n'
]

text_mode_AT = "AT+CMGF=1"
gps_command_AT = "AT+XLCSLSR=1,1,,,,,,,,,,"

class GetGpsTest(unittest2.TestCase):
    def __init__(self, *args, **kwargs):
        serial = MockSerial(stub_gps_data)
        self.modem = Modem(dev, bps, pin, serial, debug)
        self.gps_data = self.modem.getGPS()
        super(GetGpsTest, self).__init__(*args, **kwargs)

    def test_lat_lon(self):
        self.assertEqual(self.gps_data.latitude, expected_latitude)
        self.assertEqual(self.gps_data.longitude, expected_longitude)

    def test_AT_commands(self):
        commands = self.modem.device.inputs
        self.assertEqual(text_mode_AT, commands[0].strip())
        self.assertEqual(gps_command_AT, commands[1].strip())



from dateutil import parser as iso_parser
from datetime import datetime
from .utils import unquote


def parse_date_time(date, time):
    date_format = "%y/%m/%d %H:%M:%S"
    date_and_time = (date + " " + time)[:-3]
    return datetime.strptime(date_and_time, date_format)


class SMS(object):
    @classmethod
    def fromJson(cls, sms_json):
        return cls(
            sms_json["sms_id"],
            sms_json["status"],
            sms_json["sender"],
            iso_parser.parse(sms_json["date_received"]),
            sms_json["message"],
            sms_json["read"]
        )

    @classmethod
    def from_AT_response(cls, header, message):
        (sms_id_txt, status, sender, something, date, time) = header.split(",")

        sms_id = int(sms_id_txt.replace("+CMGL: ", ""))
        status = unquote(status)
        sender = unquote(sender)
        date = unquote(date.strip())
        time = unquote(time.strip())
        date_received = parse_date_time(date, time)
        message = message.strip()

        return cls(sms_id, status, sender, date_received, message, False)

    def __init__(self, sms_id, status, sender, date_received, message, read):
        self.sms_id = sms_id
        self.status = status
        self.sender = sender
        self.date_received = date_received
        self.message = message
        self.read = read

    def toJson(self):
        return {
            "sms_id": self.sms_id,
            "status": self.status,
            "sender": self.sender,
            "date_received": self.date_received.isoformat(),
            "message": self.message,
            "read": self.read
        }



import serial
import time
from .sms import SMS
from .gps import GPS
from .utils import (
    pairwise,
    quote
)


smsTimeout = 2
gpsTimeout = 5
ctrlZ = chr(26)


class Modem(object):
    def __init__(self, dev, bps=460800, pin=None, device=None, debug=False):
        self.debug = debug

        if device:
            self.device = device
        else:
            self.device = serial.Serial(dev, bps, timeout=1)

        self.setTextMode()
        if pin:
            self.unlockSIM(pin)

    # -- serial interaction via AT commands etc, maybe should be separate class
    def raw(self, message):
        self.log(message)
        self.device.write(message)

    def AT(self, separator, message):
        self.raw("AT%s%s\r\n" % (separator, message))

    def Command(self, command, args=[], sep="+"):
        if not args:
            self.AT(sep, command)
        else:
            self.AT(sep, "%s=%s" % (command, ",".join(args)))

    def Query(self, query, sep="+"):
        self.AT(sep, "%s?" % (query))

    def Read(self):
        response = self.device.readlines()

        if self.debug:
            self.log(response)

        return response
    # --------------------------------------------------------------------------

    def setTextMode(self):
        self.Command("CMGF", [str(1)])

    def unlockSIM(self, pin):
        self.Query("CPIN")
        responses = self.Read()
        pinNeeded = True
        for r in responses:
            if "READY" in r:
                pinNeeded = False

        if pinNeeded:
            self.Command("CPIN", [quote(pin)])

    def sendSMS(self, number, message):
        self.Command("CMGS", [quote(number)])
        self.raw(message)
        self.raw(ctrlZ)
        time.sleep(smsTimeout)
        return self.Read()

    def deleteSMS(self, index):
        self.Command("CMGD", [str(index)])

    def deleteAllSMS(self):
        self.Command("CMGD", [str(1), str(4)])

    def getGPS(self):
        self.Command("XLCSLSR",
                     [str(1), str(1), "", "", "", "", "", "", "", "", "", ""])

        attempts = 0

        while attempts < 10:
            time.sleep(gpsTimeout)
            raw_input = self.Read()
            raw_gps = [l for l in raw_input if l.strip() and l.strip() != "OK"]
            for line in raw_gps:
                if "XLCSLSR" in line and "XLCSLSR: request" not in line:
                    return GPS(line)
            attempts += 1

        return None

    def getSMS(self):
        self.Command("CMGL", [quote("ALL")])
        raw_input = self.Read()
        raw_sms = [l for l in raw_input if l.strip() and l.strip() != "OK"]

        messages = []

        for header, content in pairwise(raw_sms):
            message = SMS.from_AT_response(header, content)
            messages.append(message)

        return messages

    def listCommands(self):
            self.Command("CLAC")
            return self.Read()

    def log(self, message):
        if self.debug:
            print(message)



from setuptools import setup

current_version = '0.6'

setup(
    name = 'em73xx',
    packages = [ 'em73xx', 'em73xx.test' ],
    version = current_version,
    description = 'python wrapper for the em73xx series of 4G modems present in recent Lenovo Thinkpads',
    author = 'Sean McLemon',
    author_email = 'sean.mclemon@gmail.com',
    url = 'https://github.com/smcl/py-em73xx',
    download_url = 'https://github.com/smcl/py-em73xx/tarball/%s' % (current_version),
    keywords = ['thinkpad', 'em7345', 'em73xx'],
    classifiers = [],
    test_suite='em73xx.test.all',
    install_requires=[
        'unittest2',
        'pyserial',
        'python-dateutil'
    ],
    setup_requires=[
        'unittest2',
        'pyserial',
        'python-dateutil'
    ],

)



from em73xx import Modem
from .mock_serial import MockSerial
import unittest2

dev = "/dev/ttyACM0"
bps = 460800
pin = None
debug = False

# message information
recipient = "+420775123456"
content = "test"

# at commands we expect
text_mode_AT = "AT+CMGF=1"
sms_number_AT = "AT+CMGS=\"%s\"" % (recipient)
sms_content_AT = content
end_AT = "\x1a"


class SendSmsTest(unittest2.TestCase):
    def __init__(self, *args, **kwargs):
        serial = MockSerial([])
        self.modem = Modem(dev, bps, pin, serial, debug)
        self.received_sms_messages = self.modem.sendSMS(recipient, content)
        super(SendSmsTest, self).__init__(*args, **kwargs)

    def test_AT_commands(self):
        self.assertEqual(text_mode_AT, self.modem.device.inputs[0].strip())
        self.assertEqual(sms_number_AT, self.modem.device.inputs[1].strip())
        self.assertEqual(sms_content_AT, self.modem.device.inputs[2].strip())
        self.assertEqual(end_AT, self.modem.device.inputs[3].strip())
